// Checker created by Nikolay Budin

#ifdef DEBUG
#  define _GLIBCXX_DEBUG
#endif
#include <bits/stdc++.h>
#include <ext/pb_ds/assoc_container.hpp>
#include "testlib.h"
#define ff first
#define ss second
#define szof(x) ((int)x.size())
#ifndef LOCAL
#  define cerr __get_ce
#endif

using namespace std;
typedef long long ll;
typedef long double ld;
typedef pair<int, int> pii;
typedef unsigned long long ull;

using namespace __gnu_pbds;
template <typename T> using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;
template <typename K, typename V> using ordered_map = tree<K, V, less<K>, rb_tree_tag, tree_order_statistics_node_update>;

int const INF = (int)1e9 + 1e3;
ll const INFL = (ll)1e18 + 1e6;

int dx[4] = {1, 0, -1, 0};
int dy[4] = {0, 1, 0, -1};

bool is_cap(char c) {
	return 'A' <= c && c <= 'Z';
}

int main(int argc, char* argv[]) {
	setName("compare two outputs");
	registerTestlibCmd(argc, argv);

	int n = inf.readInt();
	int m = inf.readInt();

	vector<string> field;
	vector<vector<int>> color(n, vector<int>(m, -1));

	for (int i = 0; i < n; ++i) {
		field.push_back(inf.readToken());
	}

	int couches_cnt = 0;
	vector<vector<pii>> couches;
	{
		vector<vector<bool>> used(n, vector<bool>(m));

		function<void(int, int, int)> dfs = [&](int x, int y, int col) {
			used[x][y] = true;
			color[x][y] = col;
			for (int d = 0; d < 4; ++d) {
				int nx = x + dx[d];
				int ny = y + dy[d];
				if (0 <= nx && nx < n && 0 <= ny && ny < m && field[nx][ny] == field[x][y] && !used[nx][ny]) {
					dfs(nx, ny, col);
				}
			}
		};

		for (int i = 0; i < n; ++i) {
			for (int j = 0; j < m; ++j) {
				if (!used[i][j] && field[i][j] != '.') {
					dfs(i, j, couches_cnt);
					++couches_cnt;
					if (i + 1 < n && field[i][j] == field[i + 1][j]) {
						if (is_cap(field[i][j])) {
							couches.push_back({{i, j}, {i + 1, j}, {i, j + 1}, {i + 1, j + 1}});
						} else {
							couches.push_back({{i, j}, {i + 1, j}, {i, j - 1}, {i + 1, j - 1}});
						}
					} else if (j + 1 < m && field[i][j] == field[i][j + 1]) {
						if (is_cap(field[i][j])) {
							couches.push_back({{i, j}, {i, j + 1}, {i + 1, j}, {i + 1, j + 1}});
						} else {
							couches.push_back({{i, j}, {i, j + 1}, {i - 1, j}, {i - 1, j + 1}});
						}
					} else {
						assert(false);
					}

					sort(couches.back().begin(), couches.back().end());
				}
			}
		}
	}

	auto check_outp = [&](auto& stream) {
		int res = stream.readInt();
		vector<string> res_field;
		for (int i = 0; i < n; ++i) {
			res_field.push_back(stream.readToken(format("[0-9.]{%d}", m)));
		}

		map<int, int> colors_cnt;
		vector<vector<bool>> used(n, vector<bool>(m));
		int minx, miny, maxx, maxy;
		vector<pii> visited;

		function<void(int, int)> dfs = [&](int x, int y) {
			visited.push_back({x, y});
			used[x][y] = true;
			colors_cnt[color[x][y]]++;
			minx = min(minx, x);
			miny = min(miny, y);
			maxx = max(maxx, x);
			maxy = max(maxy, y);
			for (int d = 0; d < 4; ++d) {
				int nx = x + dx[d];
				int ny = y + dy[d];
				if (0 <= nx && nx < n && 0 <= ny && ny < m && res_field[nx][ny] == res_field[x][y] && !used[nx][ny]) {
					dfs(nx, ny);
				}
			}
		};

		int unfolded_cnt = 0;
		int cur_couches_cnt = 0;

		for (int i = 0; i < n; ++i) {
			for (int j = 0; j < m; ++j) {
				if (!used[i][j] && res_field[i][j] != '.') {
					colors_cnt.clear();
					minx = miny = INF;
					maxx = maxy = -INF;
					visited.clear();
					dfs(i, j);
					stream.ensuref(maxx - minx <= 1 && maxy - miny <= 1 && 
						(maxx - minx + 1) * (maxy - miny + 1) == szof(visited) && 
						(szof(visited) == 2 || szof(visited) == 4), 
						"component of the wrong shape (contains cell (%d, %d))", i + 1, j + 1);

					if (szof(visited) == 2) {
						stream.ensuref(szof(colors_cnt) == 1 && colors_cnt.begin()->ff != -1, 
							"couch of size 2 doesn't completely coincide with one of the initial ones (contains cell (%d, %d))", i + 1, j + 1);
					} else {
						stream.ensuref(szof(colors_cnt) == 2 && colors_cnt[-1] == 2,
							"wrong couch of size 4 (contains cell (%d, %d))", i + 1, j + 1);

						int couch_ind = (--colors_cnt.end())->ff;
						sort(visited.begin(), visited.end());
						stream.ensuref(visited == couches[couch_ind], "couch was unfolded in the wrong direction (contains cell (%d, %d))", i + 1, j + 1);
						++unfolded_cnt;
					}

					++cur_couches_cnt;
				}
			}
		}

		stream.ensuref(cur_couches_cnt == couches_cnt, "not all initial couches present in the output");

		stream.ensuref(unfolded_cnt == res, "number of unfolded couches isn't equal to the value in the output");

		return res;
	};

	int jans = check_outp(ans);
	int pans = check_outp(ouf);

	if (jans > pans) {
		quitf(_wa, "participant unfolded %d couches, but jury did %d", pans, jans);
	}

	if (pans > jans) {
		quitf(_fail, "participant unfolded %d couches, but jury did %d", pans, jans);
	}

	quitf(_ok, "%d unfolded couches", pans);

	return 0;
}